/**
 * \file  eeprom_writer.c
 *
 * \brief Application to write a blank EEPROM with a value.  This will allow 
 *        customers to set the Board ID to configure U-Boot.
 *
 *        Application Configuration:
 *
 *            Modules Used:
 *                I2C0
 *                UART0
 *
 *            Configurable parameters:
 *                NUM_BYTES_TO_WRITE
 *                EEPROM_VALUE_TO_WRITE
 *
 *            Hard-coded configuration of other parameters:
 *                Bus frequency    - 100kHz
 *                Addressing mode  - 7bit
 *                I2C Instance     - 0
 *                Slave Address    - 0x50
 *                EEPROM memory address - 0x0000
 *
 *        Limitations:
 *        With no flashed data in EEPROM, if the application tries to read from
 *        EEPROM, then the data values read would be "0xFF", which indicates an
 *        invalid EEPROM data.
 */

/*
* Copyright (C) 2018 Octavo Systems, LLC - http://www.octavosystems.com/ 
*/
/*
* Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
*/
/* 
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*
*    Redistributions of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

#include "hw_types.h"
#include "hw_control_AM335x.h"
#include "hw_cm_wkup.h"
#include "watchdog.h"
#include "evmAM335x.h"
#include "hsi2c.h"
#include "uartStdio.h"
#include "soc_AM335x.h"
#include "stdio.h"


/******************************************************************************
**                      INTERNAL MACRO DEFINITIONS
*******************************************************************************/

//
// MODIFY THIS CODE
//   - NUM_BYTES_TO_WRITE should match the number of data values provided in EEPROM_VALUE_TO_WRITE
//   - NUM_BYTES_TO_WRITE should not exceed BUFFER_SIZE
//

#define  NUM_BYTES_TO_WRITE            (16)
#define  EEPROM_VALUE_TO_WRITE         {0xAA, 0x55, 0x33, 0xEE, 0x41, 0x33, 0x33, 0x35, 0x42, 0x4E, 0x4C, 0x54, 0x4F, 0x53, 0x30, 0x30}

//
// END MODIFY THIS CODE
//


/* I2C address of AT24C256 eeprom */
#define  I2C_SLAVE_ADDR                (0x50)

/* Higher byte address (i.e A8-A15) */
#define  E2PROM_ADDR_MSB               (0x00)

/* Lower byte address (i.e A0-A7) */
#define  E2PROM_ADDR_LSB               (0x00)

/* I2C instance	*/
#define  I2C_0                         (0x0u)

/* System clock fed to I2C module - 48Mhz	*/
#define  I2C_SYSTEM_CLOCK              (48000000u)

/* Internal clock used by I2C module - 12Mhz	*/
#define  I2C_INTERNAL_CLOCK            (12000000u)

/* I2C bus speed or frequency - 100Khz	*/
#define	 I2C_OUTPUT_CLOCK              (100000u)

/* I2C interrupt flags to clear	*/
#define  I2C_INTERRUPT_FLAG_TO_CLR     (0x7FF)

/* Buffer Size */
#define  BUFFER_SIZE                   (1024)


/******************************************************************************
**                      INTERNAL FUNCTION PROTOTYPES
*******************************************************************************/
static void setup_platform(void);
static void print_data(unsigned char *data, unsigned int size);
static void setup_I2C(void);
static void setup_UART(void);
static void SetupI2CTransmit(unsigned int dcount);
static void SetupReception(unsigned int dcount);


/******************************************************************************
**                      INTERNAL VARIABLE DEFINITIONS
*******************************************************************************/
volatile unsigned char dataToSlave[BUFFER_SIZE];
volatile unsigned char dataFromSlave[BUFFER_SIZE];
volatile unsigned int  tCount;
volatile unsigned int  rCount;


/******************************************************************************
**              FUNCTION DEFINITIONS
******************************************************************************/

int main(void)
{
    volatile unsigned char data[NUM_BYTES_TO_WRITE] = EEPROM_VALUE_TO_WRITE;
    volatile unsigned int  i, j;

    // Setup the platform
    setup_platform();

    // Initialize variables
    for (i = 0; i < (NUM_BYTES_TO_WRITE + 2); i++) {
        dataToSlave[i] = 0;
        dataFromSlave[i] = 0;
    }

    UARTPuts("Octavo Systems EEPROM Writer v0.1\n", -1);

    // Write data to the EEPROM
    //   - Setup Address
    //   - Copy data to write
    //   - Transmit data
    //
    UARTPuts("Writing value to EEPROM\n", -1);
    tCount         = 0;
    dataToSlave[0] = E2PROM_ADDR_MSB;
    dataToSlave[1] = E2PROM_ADDR_LSB;
    for (i = 0; i < NUM_BYTES_TO_WRITE; i++) {dataToSlave[i + 2] = data[i];}
    print_data((unsigned char *)&dataToSlave[2], NUM_BYTES_TO_WRITE);

    SetupI2CTransmit(NUM_BYTES_TO_WRITE + 2);

    UARTPuts("Done writing.\n", -1);

    // Wait for EEPROM to finish writing
    j = 0;
    for (i = 0; i < 1000000; i++) {
      j++;
    }

    // Read data back from EEPROM
    //   - Use the address from the current dataToSlave buffer
    rCount = 0;
    tCount = 0;

    SetupReception(NUM_BYTES_TO_WRITE);

    // Print data from EEPROM
    UARTPuts("Value Written:\n", -1);
    print_data((unsigned char *)dataFromSlave, NUM_BYTES_TO_WRITE);

    UARTPuts("Done.\n", -1);

    while(1);
}


/*
 * \brief This function Initializes WDT, UART, and I2C
 *
 * \param  none
 *
 * \return none
*/
void setup_platform(void)
{
    // Disable WDT
    HWREG(SOC_WDT_1_REGS + WDT_WSPR) = 0xAAAAu;
    while(HWREG(SOC_WDT_1_REGS + WDT_WWPS) != 0x00);

    HWREG(SOC_WDT_1_REGS + WDT_WSPR) = 0x5555u;
    while(HWREG(SOC_WDT_1_REGS + WDT_WWPS) != 0x00);


    /* Enable the control module */
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CONTROL_CLKCTRL) =
            CM_WKUP_CONTROL_CLKCTRL_MODULEMODE_ENABLE;

    /* UART Initialization */
    setup_UART();
    UARTStdioInit();

    /* I2C Initialization */
    setup_I2C();
}


/*
** Print data
*/
static void print_data(unsigned char *data, unsigned int size) {
    unsigned char ch[2];
    unsigned char temp;
    unsigned int  i;

    if ((BUFFER_SIZE / 4) < size) {
        // Print error message
        UARTPuts("Too much data requested \n", -1);
        return;
    }

    ch[1] = 0;

    // Print first character
    UARTPuts("[", 1);

    for (i = 0; i < size; i++) {
        // Collect the Most Significant Nibble of the data byte
        temp = ((data[i] & 0xF0) >> 4);

        if (temp < 10) {
            // UARTPrintf("%c", (temp + 0x30));
            ch[0] = temp + 0x30;
            UARTPuts(ch, 1);
        } else {
            // UARTPrintf("%c", (temp + 0x37));
            ch[0] = temp + 0x37;
            UARTPuts(ch, 1);
        }

        // Collect the Least Significant Nibble of the data byte
        temp = (data[i] & 0x0F);

        if (temp < 10) {
            // UARTPrintf("%c", (temp + 0x30));
            ch[0] = temp + 0x30;
            UARTPuts(ch, 1);
        } else {
            // UARTPrintf("%c", (temp + 0x37));
            ch[0] = temp + 0x37;
            UARTPuts(ch, 1);
        }

        if (i < (size - 1)) {
            UARTPuts(", ", 2);
        }
    }

    // Print final character
    UARTPuts("]\n", -1);
}



/******************************************************************************
**              I2C FUNCTION DEFINITIONS
******************************************************************************/

/*
 * \brief Configure I2C0 on which the PMIC is interfaced
 *
 * \param  none
 *
 * \return none
 */
void setup_I2C(void)
{
    /* Enable the clock for I2C0 */
    I2C0ModuleClkConfig();

    I2CPinMuxSetup(I2C_0);

    /* Put i2c in reset/disabled state */
    I2CMasterDisable(SOC_I2C_0_REGS);

    /* Disable auto Idle functionality */
    I2CAutoIdleDisable(SOC_I2C_0_REGS);

    /* Configure i2c bus speed to 100khz */
    I2CMasterInitExpClk(SOC_I2C_0_REGS, I2C_SYSTEM_CLOCK, I2C_INTERNAL_CLOCK,
							   I2C_OUTPUT_CLOCK);

    /* Set i2c slave address */
    I2CMasterSlaveAddrSet(SOC_I2C_0_REGS, I2C_SLAVE_ADDR);

    /* Bring I2C out of reset */
    I2CMasterEnable(SOC_I2C_0_REGS);

    while(!I2CSystemStatusGet(SOC_I2C_0_REGS));
}


/*
 * \brief Clear the status of all interrupts 
 *
 * \param  none.
 *
 * \return none
 */
void CleanupInterrupts(void)
{
    I2CMasterIntClearEx(SOC_I2C_0_REGS,  I2C_INTERRUPT_FLAG_TO_CLR);
}


/*
 * \brief Transmits data over I2C0 bus 
 *
 * \param  none
 *
 * \return none
 */
void SetupI2CTransmit(unsigned int dcount)
{
    I2CSetDataCount(SOC_I2C_0_REGS, dcount);

    CleanupInterrupts();

    I2CMasterControl(SOC_I2C_0_REGS, I2C_CFG_MST_TX);

    I2CMasterStart(SOC_I2C_0_REGS);

    while(I2CMasterBusBusy(SOC_I2C_0_REGS) == 0);

    while((I2C_INT_TRANSMIT_READY == (I2CMasterIntRawStatus(SOC_I2C_0_REGS)
                                     & I2C_INT_TRANSMIT_READY)) && dcount--)
    {
        I2CMasterDataPut(SOC_I2C_0_REGS, dataToSlave[tCount++]);

        I2CMasterIntClearEx(SOC_I2C_0_REGS, I2C_INT_TRANSMIT_READY);
    }

    I2CMasterStop(SOC_I2C_0_REGS);

    while(0 == (I2CMasterIntRawStatus(SOC_I2C_0_REGS) & I2C_INT_STOP_CONDITION));

    I2CMasterIntClearEx(SOC_I2C_0_REGS, I2C_INT_STOP_CONDITION);
}


/*
 * \brief Receives data over I2C0 bus 
 *
 * \param  dcount - Number of bytes to receive.
 *
 * \return none
 */

void SetupReception(unsigned int dcount)
{
    unsigned int num_addr_bytes = 2;

    // Transmit Address Bytes
    I2CSetDataCount(SOC_I2C_0_REGS, num_addr_bytes);

    CleanupInterrupts();

    I2CMasterControl(SOC_I2C_0_REGS, I2C_CFG_MST_TX);

    I2CMasterStart(SOC_I2C_0_REGS);

    while(I2CMasterBusBusy(SOC_I2C_0_REGS) == 0);

    while((I2C_INT_TRANSMIT_READY == (I2CMasterIntRawStatus(SOC_I2C_0_REGS)
                                     & I2C_INT_TRANSMIT_READY)) && num_addr_bytes--)
    {
        I2CMasterDataPut(SOC_I2C_0_REGS, dataToSlave[tCount++]);

        I2CMasterIntClearEx(SOC_I2C_0_REGS, I2C_INT_TRANSMIT_READY);
    }

    I2CMasterStop(SOC_I2C_0_REGS);

    while(0 == (I2CMasterIntRawStatus(SOC_I2C_0_REGS) & I2C_INT_ADRR_READY_ACESS));

    // Receive Data Bytes
    I2CSetDataCount(SOC_I2C_0_REGS, dcount);

    CleanupInterrupts();

    I2CMasterControl(SOC_I2C_0_REGS, I2C_CFG_MST_RX);

    I2CMasterStart(SOC_I2C_0_REGS);

    /* Wait till the bus if free */
    while(I2CMasterBusBusy(SOC_I2C_0_REGS) == 0);

    /* Read the data from slave of dcount */
    while((dcount--))
    {
        while(0 == (I2CMasterIntRawStatus(SOC_I2C_0_REGS) & I2C_INT_RECV_READY));

        dataFromSlave[rCount++] = I2CMasterDataGet(SOC_I2C_0_REGS);

        I2CMasterIntClearEx(SOC_I2C_0_REGS, I2C_INT_RECV_READY);
    }

    I2CMasterStop(SOC_I2C_0_REGS);

    while(0 == (I2CMasterIntRawStatus(SOC_I2C_0_REGS) & I2C_INT_STOP_CONDITION));

    I2CMasterIntClearEx(SOC_I2C_0_REGS, I2C_INT_STOP_CONDITION);
}


/******************************************************************************
**              UART FUNCTION DEFINITIONS
******************************************************************************/

/*
 * \brief This function is used to initialize and configure UART Module.
 *
 * \param  none.
 *
 * \return none
*/

void setup_UART(void)
{
    volatile unsigned int regVal;

    /* Enable clock for UART0 */
    regVal = (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_UART0_CLKCTRL) &
                    ~(CM_WKUP_UART0_CLKCTRL_MODULEMODE));

    regVal |= CM_WKUP_UART0_CLKCTRL_MODULEMODE_ENABLE;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_UART0_CLKCTRL) = regVal;

    UARTStdioInit();
}































